#!/usr/bin/env perl

use strict;
use warnings;
use File::Spec;
use FindBin qw/$RealBin/;
use lib "$RealBin/../lib";

use Test::More tests=>1;
use Mash;

pass("Placeholder for when we make sketches in perl");

